/* main.c
 *
 * simple demo that scans two channel of a analog input device.
 * Scan triggers with a positive trigger at the 1st channel.
 *
 * Note: This example routine only works with a simple scan layout.
 *       Two channels with the same storage ratio of 1:1.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "libad.h"

#ifdef _WIN32
#include <malloc.h>
#endif


void
usage ()
{
  printf ("usage: ex3 <driver> [ -r <range> ]\n"
          "  <driver>     string to pass to ad_open()\n"
          "               - will prompt for name\n"
          "  <range>      range number of analog input\n"
         );
  exit (1);
}

void
check (int rc, const char *msg)
{
  /* For sake of simplicity this examples does a
   * very limited error handling. You might want
   * to replace this code with something reasonable...
   */
  if (rc != 0)
    {
      fprintf (stderr, "error: rc = %d: %s\n", rc, msg);
      exit (1);
    }
}

/* Simple example for a scan with trigger condition.
   Note: This scan routine works with a simple scan layout.
         Two channels with a store mode of AD_STORE_DISCRETE and a storage ratio of 1:1.
*/

#define NUMBERSAMPLES 20
#define CHAC 2
#define TRIGGER_VALUE 2.5

int
run_test (const char *driver)
{
  int j, adh, rc, scan_result;
  unsigned int data;

  struct ad_scan_cha_desc chav[CHAC];
  struct ad_scan_desc sd;
  struct ad_scan_state state;
  struct ad_sample_layout layout;
  uint64_t pos;
  float val;
  uint32_t nval;

  printf("Device %s\n", driver);
  adh = ad_open (driver);
  if (adh == -1)
    {
      if (ad_invalid_driver_version (errno))
        printf ("failed to open %s: invalid driver version\n", driver);
      else
        printf ("failed to open %s: err = %d\n", driver, errno);

      return 1;
    }


  /* need to ensure everything in chav is zero */
  memset(&chav, 0, sizeof(chav));

  chav[0].cha = AD_CHA_TYPE_ANALOG_IN | 1;
  chav[0].store = AD_STORE_DISCRETE;
  chav[0].ratio = 1;

  rc = ad_find_best_range (adh, chav[0].cha, &chav[0].range, -5.0, 5.0);
  check (rc, "cannot get +/- 5V range");

  /* setup trigger for the 1st channel:
     positve trigger at the defined TRIGGER_VALUE
   */
  chav[0].trg_mode = AD_TRG_POSITIVE;
  rc = ad_float_to_sample(adh, chav[0].cha, chav[0].range, TRIGGER_VALUE, &data);
  /* Note: The data have to be placed in the lower 16-bit for all 12-bit
   *       and 16-bit devices, e.g. usb-ad, mad12a, mad16a, mad16f, im-ad25
   */
  printf("Trigger Value %8.3f = hex 0x%04x (rc=%d)\n", TRIGGER_VALUE, data>>16, rc);
  chav[0].trg_par[0] = data>>16;
  chav[0].trg_par[1] = 0;

  chav[1].cha = AD_CHA_TYPE_ANALOG_IN | 2;
  chav[1].store = AD_STORE_DISCRETE;
  chav[1].ratio = 1;

  rc = ad_find_best_range (adh, chav[1].cha, &chav[1].range, -5.0, 5.0);
  check (rc, "cannot get +/- 5V range");

  chav[1].trg_mode = AD_TRG_NONE;

  /* need to ensure everything in sd is zero */
  memset(&sd, 0, sizeof(sd));

  sd.sample_rate = 0.00100f; // Samplerate in s
  sd.prehist = (int) (1.0f/4.0f * NUMBERSAMPLES);
  sd.posthist = (int) (3.0f/4.0f * NUMBERSAMPLES);
  sd.ticks_per_run = NUMBERSAMPLES;
  sd.flags = AD_SF_SAMPLES;

  /* start scan
   */
  rc = ad_start_scan (adh, &sd, CHAC, chav);
  check (rc, "failed to start scan");

  pos = 0;
  while (1)
    {
      rc = ad_poll_scan_state (adh, &state);
      check (rc, "failed to get scan state");

      if (!(state.flags & AD_SF_SCANNING))
        break;

      rc = ad_get_sample_layout (adh, 0, &layout);
      check (rc, "failed to get sample layout");

      if (pos < layout.start)
        pos = layout.start;

      /* here we have a special case:
       * both scan channel are setup with the same scan layout: AD_STORE_DISCRETE and ratio 1:1,
       * hence the sample position layout of the two channels is the same.
       */
      for ( ; pos < (layout.start+layout.prehist_samples+layout.posthist_samples); pos++)
        {
          printf ("%s %d", pos < (layout.start+layout.prehist_samples) ? "PRE ":"POST", (unsigned) pos);

          for (j = 0; j < CHAC; j++)
            {
              nval = 1;
              ad_get_samples_f (adh, j, AD_STORE_DISCRETE, pos, &nval, &val);
              printf (" %8.3f", val);
            }

          printf ("\n");
        }
    }

  rc = ad_stop_scan (adh, &scan_result);
  check (rc, "failed to stop scan");

  if (scan_result)
    printf ("Scan result = %d\n", scan_result);

  for (j = 0; j < CHAC; j++)
    {
      rc = ad_get_sample_layout (adh, j, &layout);
      check (rc, "failed to get sample layout");

      printf ("channel #%d\n", j);

      printf ("buffer start    : %d\n", (int) layout.buffer_start);
      printf ("start pos       : %d\n", (int) layout.start);
      printf ("samples prehist : %d\n", (int) layout.prehist_samples);
      printf ("samples posthist: %d\n", (int) layout.posthist_samples);

      printf ("Pre-History:\n");
      for (pos = 0; pos < layout.prehist_samples; pos++)
        {
          nval = 1;
          rc = ad_get_samples_f (adh, j, AD_STORE_DISCRETE, layout.start+pos, &nval, &val);
          check (rc, "failed to read sample");

          printf (" %8.3f", val);

          if (pos % 8 == 7)
            printf ("\n");
        }
      printf ("\n");

      printf ("Post-History:\n");
      for (pos = 0; pos < layout.posthist_samples; pos++)
        {
          /* Post history starts at the layout position start+prehist_samples
           */
          nval = 1;
          rc = ad_get_samples_f (adh, j, AD_STORE_DISCRETE, layout.start+layout.prehist_samples+pos, &nval, &val);
          check (rc, "failed to read sample");

          printf (" %8.3f", val);

          if (pos % 8 == 7)
            printf ("\n");
        }
      printf ("\n");
    }

  ad_close(adh);

  return rc;
}


/* main entry
 */

int
main (int argc, char *argv[])
{
  char *driver, *p, tmp[80];
  int rc;

  if (argc <= 1)
    usage ();

  if (argv[1][0] == '-')
    {
      printf ("driver to open: ");
      fgets (tmp, sizeof(tmp), stdin);

      p = strchr (tmp, '\n');
      if (p)
        *p = '\0';

      driver = tmp;
    }
  else
    driver = argv[1];

  rc = run_test (driver);

  if (rc)
    printf ("Error code rc=%d\n",rc);

  if (argv[1][0] == '-')
    {
      printf ("press return to continue...\n");
      fgets (tmp, sizeof(tmp), stdin);
    }

  return rc;
}
